/*
 *
 *  Copyright (C) THL A29 Limited, a Tencent company. All rights reserved.
 *  SPDX-License-Identifier: Apache-2.0
 *
 */

import React, { useCallback, useEffect, useRef, useState } from 'react';
import { Button, Form, Input } from 'tea-component';
import { useForm } from 'react-hook-form';
import { SignatureConfirmModal } from '../utils/common';
import { useLocation, useNavigate } from 'react-router-dom';
import { SendRequestParam } from '../event-page';
import { accountSign, createContract, invokeContract, preCheckForSignature } from '../utils/utils';
import { useChainStore } from './popup';
import GlossaryGuide from '../utils/glossary-guide';
import chainStorageUtils from '../utils/storage';
import { ContractLogo } from '../components/contract-logo';
import { Account } from '../utils/interface';
import { getNowSeconds, sendMessageToContentScript } from '../utils/tools';



function AccountSignature() {
    const location = useLocation();
    const contractRequest = location.state as SendRequestParam;
    const navigate = useNavigate();
    const {
        control,
        setValue,
        formState: { isValid },
    } = useForm({
        mode: 'onBlur',
    });
    const { currentTab, currentAccount, chains } = useChainStore();
    const [chainId, setChainId] = useState<string>();
    const [account, setAccount] = useState<Account>();
    const [loading, setLoading] = useState(false);

    const signatureConfirmRef = useRef();
    const sendMessage = useCallback((info)=>{
        const contractRequest = location.state as SendRequestParam;
        sendMessageToContentScript({
            operation: contractRequest.operation,
            ticket: contractRequest.ticket,
            data: {
                status: 'done',
                timestamp: getNowSeconds(),
                info,
            },
        });
    },[]);
    const onSubmit = useCallback(async () => {
        const doConfirm = async () => {
            try {
                setLoading(true);
                const signCode = await accountSign(account, contractRequest.body);
                sendMessage({
                    code:0,
                    res:signCode 
                })
                chainStorageUtils.setLastTransTime();
                navigate('/');
            } catch (e) {
                sendMessage({
                    code:2,
                    res:'插件异常退出'
                })
            } finally {
                setLoading(false);
            }
        };
        if ((await chainStorageUtils.getLastTransTime())) {
            doConfirm();
        } else {
            // @ts-ignore
            signatureConfirmRef.current.show({
                confirm: doConfirm,
            });
        }

    }, [contractRequest, account]);

    const backToHome = useCallback(() => {
        navigate("/");
    }, []);

    const onCancel = useCallback(()=>{
        sendMessage({
            code:1,
            res:'插件取消验签'
        })
        backToHome();
    },[contractRequest]);
    useEffect(() => {
        const contractRequest = location.state as SendRequestParam;
        const {chainId,accountAddress,body} = contractRequest;
        if (!chainId || !accountAddress) {
            navigate('/', {
                state: {
                    alert: '缺少链ID或者用户地址'
                }
            })
            sendMessage({
                code:2,
                res:'缺少链ID或者用户地址'
            })
            return;
        }
        if(!body){
            navigate('/', {
                state: {
                    alert: '缺少验签随机字符串'
                }
            })
            sendMessage({
                code:2,
                res:'缺少验签随机字符串'
            })
            return;
        }
        chainStorageUtils.getChainAccounts(chainId).then(accounts => {
            const account = accounts.find(ac => ac.address === accountAddress);
            if (!account) {
                sendMessage({
                    code:2,
                    res:'您指定的区块链网络或者链账户未添加到插件里，请检查信息后重试。'
                })
                navigate('/', {
                    state: {
                        alert: '您指定的区块链网络或者链账户未添加到插件里，请检查信息后重试。'
                    }
                })
                return;
            }
            setChainId(chainId);
            setAccount(account);
        });
    }, [])


    return <div className={'signature'}>
        <div className='page-title'>请求签名</div>
        <div className="current-web-info">
            <img src={currentTab?.favIconUrl} />
            <div className="current-web-addr">
                {currentTab?.host}
            </div>
        </div>
        <Form layout={'vertical'}>
            <Form.Item
                label={<GlossaryGuide title={'待签名信息'} />}>
                <Input
                    size={'full'}
                    readOnly
                    multiline
                    value={contractRequest?.body}
                />
            </Form.Item>

            <Form.Item
                label={<GlossaryGuide title={'指定签名账户'} />}>
                <Input
                    readOnly
                    size={'full'}
                    value = {chainId}
                />
            </Form.Item>

            <Form.Item>
                <Input
                    readOnly
                    size={'full'}
                    value = {account?.name}
                />
            </Form.Item>
        </Form>
        <div className={'flex-grow'} />
        <div className="page-options">
            <div className="connect-concel connect-bt" onClick={onCancel}>
                取消
            </div>
            <Button type={'primary'} className='connect-sub connect-bt' disabled={!isValid || !Boolean(contractRequest)}
                onClick={onSubmit} loading={loading}>确定签名</Button>
        </div>
        <SignatureConfirmModal ref={signatureConfirmRef} />

    </div>;
}

export default AccountSignature;

