/*
 *
 *  Copyright (C) THL A29 Limited, a Tencent company. All rights reserved.
 *  SPDX-License-Identifier: Apache-2.0
 *
 */

import React, { useEffect, useState } from 'react';
import './account.less';
import ChainsPage from './chains-page';
import { MemoryRouter as Router, NavigateFunction, Route, Routes, useLocation, useNavigate } from 'react-router-dom';
import LoginPage from './login-page';
import ChainNewPage from './chain-new-page';
import ChainDetailPage from './chain-detail-page';
import chainStorageUtils from '../utils/storage';
import AccountsPage from './accounts-page';
import AccountNewPage from './account-new-page';
import SignaturePage from './signature-page';
import TxLogsPage from './tx-logs-page';
import SettingPage from './setting-page';
import LoginLockPage from './lock-page';
import HomePage from './home/home-page';
import { Account, Chain } from '../utils/interface';
import { ChainPage } from '../utils/common';
import create from 'zustand';
import AboutPage from './about-page';
import SubscribeContractPage from './subscribe-contract-page';
import { ContractDetailPage } from './subscribe-detail/contract-detail';
import { TransactionDetailPage } from './subscribe-detail/transaction-detail';
import AccountConnect from './account-connect';
import AccountSignature from './account-signature';
import AccountConnectSetting from './account-connect-setting';
import '../iconsvg/index';
import { SendRequestParam } from '../event-page';
import { responseAccountInfo } from '../utils/utils';
import { getNowSeconds, sendMessageToContentScript } from '../utils/tools';
import deepEqual from 'deep-equal';
import ChainImportPage from './chain-import-page';
import AccountImportPage from './account-import-page';
import { checkUpdateExpiredOfficialChainData } from '../utils/official-chain';

/**
 * @description
 */
chrome.runtime.connect({ name: 'popup' });

const initState: {
  chains: Chain[];
  selectedChain: Chain;
  currentAccount: Account;
  initialized: boolean;
  currentTab: chrome.tabs.Tab;
} = {
  chains: [],
  selectedChain: null,
  currentAccount: null,
  /**
   *@description 初始化完成标记位，用户从登录页进入添加网络页面，完成首次网络添加后，标记位恢复为true
   */
  initialized: true,
  currentTab: null,
};

const OPERATION_NAV_MAP = {
  createUserContract: '/signature',
  invokeUserContract: '/signature',
  openConnect: '/accounts/connect',
  openAccountSignature: '/accounts/signature',
  importChainConfig: '/chains/import',
  importAccountConfig: '/accounts/import',
};

export const navByMessage = (nav: NavigateFunction, temp: SendRequestParam) => {
  const navPath = OPERATION_NAV_MAP[temp.operation] || '/';
  nav(navPath, {
    state: temp,
  });
};

type Tab = chrome.tabs.Tab & { host?: string };

/**
 * 全局状态上下文
 */
export const useChainStore = create<{
  chains: Chain[];
  selectedChain: Chain;
  currentAccount?: Account;
  initialized: boolean;
  currentTab: Tab;
  setChains: (chains: Chain[]) => void;
  setSelectedChain: (chains: Chain) => void;
  setCurrentAccount: (account: Account) => void;
  updateSelectData: ({ chain, account }: { chain: Chain; account: Account }) => Promise<void>;
  reset: () => void;
  setInitialized: (b: Boolean) => void;
  setCurrentTab: (tab?: Tab) => void;
}>((set: any, get) => ({
  ...initState,
  setChains: (chains) =>
    set((state) => ({
      ...state,
      chains,
    })),
  async updateSelectData({ chain, account }) {
    let { currentAccount, selectedChain } = get();
    if (chain?.chainId !== selectedChain.chainId) {
      await chainStorageUtils.setSelectedChain(chain);
      selectedChain = chain;
    }
    if (account?.address !== currentAccount.address) {
      await chainStorageUtils.setCurrentAccount(account.address);
      currentAccount = account;
    }
    set((state) => ({
      ...state,
      selectedChain,
      currentAccount,
    }));
  },
  setSelectedChain: (chain) =>
    set((state) => ({
      ...state,
      selectedChain: chain,
    })),
  setCurrentAccount: (account) =>
    set((state) => {
      if (!deepEqual(account, state.currentAccount)) {
        const { chainName, chainId } = state.selectedChain || {};
        sendMessageToContentScript({
          operation: 'changeCurrentAccount',
          data: {
            status: 'done',
            timestamp: getNowSeconds(),
            info: {
              accounts: responseAccountInfo([account], state.currentTab?.host),
              chain: { chainName, chainId },
            },
          },
        });
      }
      return {
        ...state,
        currentAccount: account,
      };
    }),
  reset: () => set(() => initState),
  setInitialized: (b: boolean) =>
    set((state) => ({
      ...state,
      initialized: b,
    })),
  setCurrentTab: (tab: Tab) =>
    set((state) => ({
      ...state,
      currentTab: tab,
    })),
}));
/**
 * @description 全局上下文
 */
export const AppContext = React.createContext<{
  isLoggedIn: boolean;
}>({ isLoggedIn: false });

export default function Popup() {
  return (
    <Router>
      <CustomRoutes />
    </Router>
  );
}

function CustomRoutes() {
  const navigate = useNavigate();
  const { pathname } = useLocation();

  const { setSelectedChain, setChains, setCurrentTab, setCurrentAccount, selectedChain } = useChainStore();
  const [hasInit, setHasInit] = useState<boolean>(false);
  const [isLoggedIn, setIsLoggedIn] = useState<boolean>(false);
  useEffect(() => {
    const hideChainSelectPage = ['/login', '/login-lock', '/signature', '/accounts/connect', '/accounts/signature'];
    setIsLoggedIn(hideChainSelectPage.indexOf(pathname) === -1);
  }, [pathname]);
  useEffect(() => {
    const checkFn = async () => {
      const [loginRes, loginLifeRes, tempOperationRes] = await Promise.all([
        chainStorageUtils.getLogin(),
        chainStorageUtils.getLoginLife(),
        chainStorageUtils.getTempOperation(),
      ]);
      setHasInit(true);
      // eslint-disable-next-line @typescript-eslint/no-misused-promises
      chrome.tabs.query({ active: true }, async (tabs) => {
        const tab = tabs.find((ele) => ele.id === tempOperationRes?.tabId);
        // const tab = tabs[0];
        // eslint-disable-next-line @typescript-eslint/consistent-type-assertions
        if (!tab) setCurrentTab({} as Tab);
        // eslint-disable-next-line no-useless-escape
        const host = tab?.url?.match(/^https?:\/\/[^\/]+/)?.[0];
        if (host) {
          setCurrentTab({
            ...tab,
            host,
          });
        }
        if (!loginRes) {
          return navigate('/login');
        }
        if (!loginLifeRes) {
          await chainStorageUtils.setLoginLock();
          return navigate('/login-lock', {
            state: tempOperationRes,
          });
        }
        if (tempOperationRes) {
          navByMessage(navigate, tempOperationRes);
        } else {
          return navigate('/');
        }
      });
    };

    Promise.all([
      chainStorageUtils.getChains(),
      chainStorageUtils.getSelectedChain(),
      chainStorageUtils.getCurrentAccount(),
    ])
      .then(async ([chains, selectedChain, currentAccount]) => {
        // 检测更新过期的测试链数据
        const { correctChains, correctSelectedChain } = await checkUpdateExpiredOfficialChainData({
          chains,
          selectedChain,
        });
        setSelectedChain(correctSelectedChain);
        setChains(correctChains);
        setCurrentAccount(currentAccount);
      })
      .then(checkFn);
  }, []);

  useEffect(() => {
    chainStorageUtils.getCurrentAccount().then((account) => {
      setCurrentAccount(account);
    });
  }, [selectedChain]);

  return (
    <div className={'content'}>
      <AppContext.Provider
        value={{
          isLoggedIn,
        }}
      >
        {hasInit && (
          <ChainPage isLoggedIn={isLoggedIn}>
            <Routes>
              <Route path={'/'} element={<HomePage />} />
              <Route index element={<HomePage />} />
              {/* <Route path={'login'} element={<LoginPage onLogged={() => setIsLoggedIn(true)} />} />
          <Route path={'login-lock'} element={<LoginLockPage onClocked={() => setIsLoggedIn(true)} onReset={() => setIsLoggedIn(false)} />} /> */}
              <Route path={'login'} element={<LoginPage />} />
              <Route path={'login-lock'} element={<LoginLockPage />} />
              {/* <Route index element={<SignaturePage/>}/> */}
              <Route path={'signature'} element={<SignaturePage />} />
              <Route path={'tx-logs'} element={<TxLogsPage />} />
              <Route path={'chains'}>
                <Route index element={<ChainsPage />} />
                <Route path={'new'} element={<ChainNewPage />} />
                {/* 导入 */}
                <Route path={'import'} element={<ChainImportPage />} />
                <Route path={':id'} element={<ChainDetailPage />} />
              </Route>
              <Route path={'accounts'}>
                <Route index element={<AccountsPage />} />
                <Route path={'new'} element={<AccountNewPage />} />
                {/* 导入 */}
                <Route path={'import'} element={<AccountImportPage />} />
                <Route path={'connect'} element={<AccountConnect />} />
                <Route path={'signature'} element={<AccountSignature />} />
                <Route path={'connect-setting'} element={<AccountConnectSetting />} />
              </Route>
              <Route path={'setting'} element={<SettingPage onReset={() => setIsLoggedIn(false)} />} />
              <Route path={'about-us'} element={<AboutPage />} />

              <Route path={'subscribe-contract'}>
                <Route path={'new'} element={<SubscribeContractPage />} />
                <Route path={'contract-detail'} element={<ContractDetailPage />} />
                <Route path={'transaction-detail'} element={<TransactionDetailPage />} />
              </Route>
            </Routes>
          </ChainPage>
        )}
      </AppContext.Provider>
    </div>
  );
}
