/*
 *
 *  Copyright (C) THL A29 Limited, a Tencent company. All rights reserved.
 *  SPDX-License-Identifier: Apache-2.0
 *
 */

import React, { useCallback, useState } from 'react';
import { Button, Form, Input, message, Select } from 'tea-component';
import { Controller, useForm } from 'react-hook-form';
import { useNavigate } from 'react-router-dom';
import { symbol } from '../utils/utils';
import { useChainStore } from './popup';
import GlossaryGuide from '../utils/glossary-guide';
import { contractStoreUtils, SubscribeContractItem } from '../utils/storage';
import { DetailPage } from '../utils/common';
import { createIconToken } from '../components/contract-logo';
import { CONTRACT_TYPE } from '../config/contract';

const ContractTypeOptions = [
  { text: 'CMDFA（同质化数字资产）', value: CONTRACT_TYPE.CMDFA },
  { text: 'CMNFA（非同质化数字资产）', value: CONTRACT_TYPE.CMNFA },
  { text: 'CMID（区块链身份认证）', value: CONTRACT_TYPE.CMID },
  { text: '其他类', value: CONTRACT_TYPE.OTHER },
];

function SubScribeContractPage() {
  const {
    control,
    getValues,
    formState: { isValid },
  } = useForm({
    mode: 'onChange',
    defaultValues: {
      FTMinPrecision: 4,
    } as any,
  });

  const { selectedChain, currentAccount } = useChainStore();
  const [loading] = useState(false);
  const [contractType, setContractType] = useState<ContractType>(CONTRACT_TYPE.CMDFA);
  const [FTPerName, setFTPerName] = useState('');
  const [contractName, setContractName] = useState('');
  const navigate = useNavigate();
  const onSubmit = useCallback(async () => {
    // setLoading(true);
    const values: SubscribeContractItem = getValues();
    contractStoreUtils
      .setSubscribe(selectedChain.chainId, {
        contractType,
        FTPerName,
        contractIcon: createIconToken(),
        contractName,
        ...values,
      })
      .then(() => {
        message.success({ content: '订阅成功' });
        setTimeout(() => {
          navigate('/');
        }, 1000);
      })
      .catch((err) => {
        message.error({ content: `订阅失败:${err.message || err}` });
      });
  }, [selectedChain.chainId, FTPerName, contractType, contractName]);

  const getSymbol = () => {
    if (contractName && contractType === CONTRACT_TYPE.CMDFA) {
      symbol({ contractName, account: currentAccount, chainId: selectedChain.chainId }).then(
        (res: string | undefined) => {
          if (res) {
            setFTPerName(res);
          } else {
            message.error({ content: '查询不到该合约，请检查您输入的信息是否正确，合约状态是否正常。' });
          }
        },
      );
    }
  };

  // const currentValues = getValues();
  return (
    <DetailPage title={'订阅合约'} backUrl={'/'}>
      <Form layout={'vertical'} className={'mt-4n'}>
        <Form.Item label={<GlossaryGuide title={'合约类型'} />}>
          <Select
            size={'full'}
            matchButtonWidth
            value={contractType}
            onChange={(type: ContractType) => setContractType(type)}
            appearance="button"
            options={ContractTypeOptions}
          />
        </Form.Item>
        <Form.Item
          extra="请确保该合约已经部署到当前链上，否则将获取不到数据"
          label={<GlossaryGuide title={'合约名称'} />}
        >
          <Input size={'full'} onBlur={getSymbol} onChange={setContractName} />
        </Form.Item>

        {contractType === CONTRACT_TYPE.CMDFA && (
          <>
            <Form.Item label={<GlossaryGuide title={'FT简称'} />}>
              <Input size={'full'} value={FTPerName} onChange={setFTPerName} />
            </Form.Item>
            <Controller
              control={control}
              name="FTMinPrecision"
              render={({ field }) => (
                <Form.Item label={<GlossaryGuide title={'最小精度'} />}>
                  <Input
                    type="number"
                    step={1}
                    min={1}
                    max={8}
                    size={'full'}
                    {...field}
                    onChange={(nextVal) => {
                      field.onChange(Math.max(Math.min(parseInt(nextVal.charAt(nextVal.length - 1), 10) || 1, 8), 1));
                    }}
                  />
                </Form.Item>
              )}
            />
          </>
        )}
      </Form>

      <footer className={'mt-6n'}>
        <Button
          type={'primary'}
          className={'btn-lg'}
          disabled={!isValid || (contractType === CONTRACT_TYPE.CMDFA && (!FTPerName || !contractName))}
          onClick={() => {
            onSubmit();
          }}
          loading={loading}
        >
          确定订阅
        </Button>
      </footer>
    </DetailPage>
  );
}

export default SubScribeContractPage;
