/*
 *
 *  Copyright (C) THL A29 Limited, a Tencent company. All rights reserved.
 *  SPDX-License-Identifier: Apache-2.0
 *
 */

import { default as localforage } from 'localforage';
import { Account, Chain, PluginSetting, TxLog } from './interface';
import { v4 as uuidv4 } from 'uuid';
import { CreateUserContractRequest, InvokeUserContractRequest, SendRequestParam } from '../event-page';
import md5 from 'md5';
import { CONTRACT_TYPE_REPLACE_MAP } from '../config/contract';

function binaryStringToFile(fileStr: string) {
  const array = new Uint8Array(fileStr.length);
  for (let i = 0; i < fileStr.length; i++) {
    array[i] = fileStr.charCodeAt(i);
  }
  return new File([new Blob([array], { type: 'application/octet-stream' })], 'contract');
}

const chainStore = localforage.createInstance({
  name: 'chain',
  version: 1.0,
  driver: localforage.INDEXEDDB,
});

/**
 * setting用于非跟链有直接关系，不考虑存储周期的数据，比如窗体ID缓存
 */
const settingStore = localforage.createInstance({
  name: 'setting',
  version: 1.0,
  driver: localforage.INDEXEDDB,
});

/**
 * setting用于非跟链有直接关系，不考虑存储周期的数据，比如窗体ID缓存
 */
const contractStore = localforage.createInstance({
  name: 'contract',
  version: 1.0,
  driver: localforage.INDEXEDDB,
});

const headColors = [
  '#7F669D&#FBFACD',
  '#65647C&#F1D3B3',
  '#852999&#F5D5AE',
  '#3A8891&#F2DEBA',
  '#E97777&#FCDDB0',
  '#EA9518&#A5F1E9',
  '#BF760A&#F4EA2A',
  '#344D67&#F3ECB0',
  '#FF577F&#FFD384',
  '#905E96&#FFD372',
  '#874C62&#A7D2CB',
  '#AC4425&#F0F2B6',
];

export const getColor = async () => {
  const color = await settingStoreUtils.getColor();
  // let newColor;
  // if (!color) {
  //   const colorNumber1 = Math.floor(Math.random() * 255);
  //   const colorNumber2 = Math.floor(Math.random() * 255);
  //   const colorNumber3 = Math.floor(Math.random() * 255);
  //   newColor = Color.rgb(colorNumber1, colorNumber2, colorNumber3).hex();
  // } else {
  //   const arr = Color(color).rgb().array()
  //   newColor = Color.rgb((arr[0] + 135) % 255, (arr[1] + 141) % 255, (arr[2] + 147) % 255).hex();
  // }
  let index = headColors.indexOf(color);
  if (index > -1 && index < 11) {
    index += 1;
  } else {
    index = 0;
  }
  const newColor = headColors[index];
  settingStoreUtils.setColor(newColor);
  return newColor;
};
/**
 * 默认配置
 */
export const initSettingValues: PluginSetting = {
  proxyHostname: CHAIN_MAKER.proxyServerURL,
  proxyHostnameTls: CHAIN_MAKER.proxyServerURL2,
  /**
   * @description 分钟
   */
  lockLife: 30,
};

const localForgeUtils = {
  /**
   * @param expire 单位是秒
   */
  setData: async <T>(store: LocalForage, key: string, data: T, expire: number | false) => {
    await store.setItem(key, {
      expire: typeof expire === 'number' ? Date.now() + expire * 1000 : expire,
      data,
    });
  },

  getData: async <T>(store: LocalForage, key: string): Promise<any> => {
    const expireData: {
      expire: number;
      data: T;
    } = await store.getItem(key);
    if (expireData) {
      if (typeof expireData.expire === 'number' && expireData.expire < Date.now()) {
        await store.removeItem(key);
        return null;
      }
      return expireData.data;
    }
    return null;
  },
  // eslint-disable-next-line @typescript-eslint/no-unused-vars
  removeData: <T>(store: LocalForage, key: string) => {
    store.removeItem(key);
  },
};

export default class chainStorageUtils {
  /**
   * @param expire 单位是秒
   */
  // eslint-disable-next-line @typescript-eslint/no-unused-vars
  static async setData<T>(key: string, data: T, expire: number | false) {
    await localForgeUtils.setData(chainStore, key, data, expire);
  }
  // eslint-disable-next-line @typescript-eslint/no-unused-vars
  static async getData<T>(key: string) {
    return await localForgeUtils.getData(chainStore, key);
  }

  // eslint-disable-next-line @typescript-eslint/no-unused-vars
  static async removeData<T>(key: string) {
    await localForgeUtils.removeData(chainStore, key);
  }

  /**
   * 添加链
   * @param chain
   */
  static async addChain(chain: Chain): Promise<Chain[]> {
    const chains = ((await chainStorageUtils.getData('chain_list')) as Chain[]) ?? [];
    const newChains = [chain, ...chains];
    await chainStorageUtils.setData('chain_list', newChains, false);
    return newChains;
  }

  /**
   * 获取链列表
   */
  static async getChains() {
    return ((await chainStorageUtils.getData('chain_list')) as Chain[]) ?? [];
  }
  /**
   * 根据chainId获取chain详情
   * @param chainId
   * @returns
   */
  static async getChain(chainId: Chain['chainId']): Promise<Chain | undefined> {
    const chains = ((await chainStorageUtils.getData('chain_list')) as Chain[]) ?? [];
    if (chains.length) {
      return chains.find((item) => item.chainId === chainId);
    }
  }

  static async setChains(chains: Chain[]) {
    return await chainStorageUtils.setData('chain_list', chains, false);
  }

  static async deleteChain(index: number) {
    const chains = ((await chainStorageUtils.getData('chain_list')) as Chain[]) ?? [];
    const chainsDeleted = chains.splice(index, 1);
    await chainStorageUtils.setData('chain_list', chains, false);
    chainsDeleted.forEach((item) => {
      Promise.all([contractStoreUtils.abortSubscribe(item.chainId), chainStorageUtils.emptyChainAccount(item.chainId)]);
    });
    return chains;
  }

  static async updateChain(chainId: Chain['chainId'], p: Partial<Chain>) {
    const chains = ((await chainStorageUtils.getData('chain_list')) as Chain[]) ?? [];
    let updatedChain: Chain;
    chains.forEach((item) => {
      if (item.chainId === chainId) {
        Object.keys(p).forEach((k) => {
          item[k] = p[k];
        });
        updatedChain = item;
      }
    });
    await chainStorageUtils.setChains(chains);
    return { updatedChain, chains };
  }

  static async uploadFile(item: File): Promise<string> {
    if (!(item instanceof File)) {
      return null;
    }
    const id = uuidv4();
    await chainStore.setItem(`file_${id}`, item);
    return id;
  }

  static async uploadFiles(items: File[]): Promise<string[]> {
    const ids = await Promise.all(items.map((item) => chainStorageUtils.uploadFile(item)));
    return ids;
  }

  static async getUploadFile(id: string): Promise<File | null> {
    return await chainStore.getItem(`file_${id}`);
  }

  static async getUploadFiles(ids: string[]): Promise<(File | null)[]> {
    return await Promise.all(ids.map((item) => chainStorageUtils.getUploadFile(item)));
  }

  /**
   * 登录信息
   */
  static async setLogin(password): Promise<void> {
    await chainStorageUtils.setData('login_token', md5(password), false);
    return;
  }

  static async getLogin(): Promise<string | null> {
    return chainStorageUtils.getData('login_token');
  }

  /**
   * 用户关闭插件时，记录该时间，重新唤起插件，来判断是否过期， 如果重新输入密码激活
   */
  static async setLoginLife(): Promise<void> {
    const setting = await chainStorageUtils.getSetting();
    await chainStorageUtils.setData('login_life', true, setting.lockLife * 60);
    return;
  }

  static async getLoginLife(): Promise<string> {
    return await chainStorageUtils.getData<string>('login_life');
  }

  /**
   * @description true表示锁屏
   */
  static async setLoginLock() {
    await chainStorageUtils.setData('login_lock', true, false);
    return;
  }

  static async removeLoginLock() {
    await chainStorageUtils.removeData('login_lock');
    return;
  }

  static async getLoginLock() {
    return await chainStorageUtils.getData<boolean>('login_lock');
  }

  /**
   * 记录签名记录时间
   */
  static async setLastTransTime() {
    const setting = await chainStorageUtils.getSetting();
    await chainStorageUtils.setData('last_signature_time', Date.now(), setting.lockLife * 60);
  }

  static async getLastTransTime() {
    return await chainStorageUtils.getData('last_signature_time');
  }

  static async addChainAccount(chainId: string, account: Account) {
    account.color = await getColor();
    const accounts = await chainStorageUtils.getChainAccounts(chainId);
    const newAccounts = [account, ...accounts];
    await chainStorageUtils.setData(`chain_account_${chainId}`, newAccounts, false);
    return newAccounts;
  }

  static async emptyChainAccount(chainId: string) {
    await chainStorageUtils.removeData(`chain_account_${chainId}`);
  }

  static async deleteChainAccount(chainId: string, index: number) {
    const accounts = (await chainStorageUtils.getData(`chain_account_${chainId}`)) as Account[];
    accounts.splice(index, 1);
    await chainStorageUtils.setData(`chain_account_${chainId}`, accounts, false);
    return accounts;
  }
  /**
   * 获取链下所有账号
   * @param chainId
   * @returns
   */
  static async getChainAccounts(chainId: string) {
    return ((await chainStorageUtils.getData(`chain_account_${chainId}`)) as Account[]) ?? [];
  }

  static async setChainAccount(chainId: string, accounts: Account[]) {
    await chainStorageUtils.setData(`chain_account_${chainId}`, accounts, false);
    return accounts;
  }

  static async getCurrentChainAccounts() {
    const chain = (await chainStorageUtils.getData(`active_chain`)) as Chain;
    if (!chain?.chainId) {
      return [];
    }
    return ((await chainStorageUtils.getData(`chain_account_${chain?.chainId}`)) as Account[]) ?? [];
  }

  static async setCurrentChainAccount(accounts: Account[]) {
    const chain = (await chainStorageUtils.getData(`active_chain`)) as Chain;
    if (!chain?.chainId) {
      return false;
    }
    await chainStorageUtils.setData(`chain_account_${chain?.chainId}`, accounts, false);
  }

  /**
   * 获取当前登录的账号，默认为链的第一个
   * @returns
   */
  static async getCurrentAccount() {
    const chain = (await chainStorageUtils.getData(`active_chain`)) as Chain;
    if (!chain?.chainId) {
      return undefined;
    }
    const { chainId } = chain;
    const accounts = ((await chainStorageUtils.getData(`chain_account_${chainId}`)) as Account[]) ?? [];
    const account = accounts.find((ac) => ac.isCurrent);
    if (account) {
      return account;
    }
    if (accounts[0]) {
      accounts[0].isCurrent = true;
      await chainStorageUtils.setData(`chain_account_${chainId}`, accounts, false);
      return accounts[0];
    }
    return undefined;
  }

  // 根据addr设置当前链帐号
  static async setCurrentAccount(address: string) {
    const chain = (await chainStorageUtils.getData(`active_chain`)) as Chain;
    if (!chain?.chainId) {
      return undefined;
    }
    const { chainId } = chain;
    const accounts = ((await chainStorageUtils.getData(`chain_account_${chainId}`)) as Account[]) ?? [];
    let resAccount;
    accounts.forEach((ac) => {
      ac.isCurrent = ac.address === address;
    });
    const account = accounts.find((ac) => ac.isCurrent);
    if (account) {
      // await chainStorageUtils.setData('log_account',account[0],false);
      resAccount = account;
    } else {
      accounts[0].isCurrent = true;
      console.error('当前链不存在这个用户！');
      resAccount = accounts[0];
    }
    await chainStorageUtils.setData(`chain_account_${chainId}`, accounts, false);
    return resAccount;
  }

  static async getTxLogs(chainId: Chain['chainId']) {
    return ((await chainStorageUtils.getData(`tx_logs_${chainId}`)) as TxLog[]) ?? [];
  }

  static async updateTxLogs(chainId: Chain['chainId'], logs: TxLog[]) {
    return await chainStorageUtils.setData(`tx_logs_${chainId}`, logs, false);
  }

  /**
   * DB中存储顺序为倒序
   * @param log
   */
  static async addTxLog(log: TxLog) {
    const logs = await chainStorageUtils.getTxLogs(log.chainId);
    await chainStorageUtils.setData(`tx_logs_${log.chainId}`, [log, ...logs], false);
  }

  /**
   * @description 针对创建合约，需要将文件临时存储
   * @param log
   */
  static async setTempOperation(log: SendRequestParam) {
    if (log.body?.contractFile) {
      log.body.contractFile = await chainStorageUtils.uploadFile(binaryStringToFile(log.body.contractFile));
    }
    // 将参数中的特殊文件内容进行转存
    if (log.body?.params?.CONTRACT_BYTECODE) {
      log.body.params.CONTRACT_BYTECODE = await chainStorageUtils.uploadFile(
        binaryStringToFile(log.body?.params?.CONTRACT_BYTECODE),
      );
    }
    await chainStorageUtils.setData(`temp_operation`, log, false);
  }

  /**
   * @description 拿出来之后就删除DB存储，只能消费一次
   */
  static async getTempOperation() {
    const item = (await chainStorageUtils.getData(`temp_operation`)) as unknown as
      | CreateUserContractRequest
      | InvokeUserContractRequest
      | SendRequestParam;
    if (item) {
      await chainStorageUtils.removeData('temp_operation');
    }
    return item;
  }

  static async setActiveTabId(tabId: number) {
    await chainStorageUtils.setData(`active_tab_id`, tabId, false);
  }

  static async getActiveTabId() {
    return (await chainStorageUtils.getData(`active_tab_id`)) as number;
  }

  static async getSetting(): Promise<PluginSetting> {
    const setting = await chainStorageUtils.getData<PluginSetting>('setting');
    if (setting === null) {
      await chainStorageUtils.setSetting(initSettingValues);
      return initSettingValues;
    }
    return setting;
  }

  static async setSetting(setting: PluginSetting) {
    return await chainStorageUtils.setData('setting', setting, false);
  }

  static clearData() {
    return chainStore.clear();
  }

  static async setSelectedChain(chainId: Chain) {
    await chainStorageUtils.setData(`active_chain`, chainId, false);
  }

  static async getSelectedChain() {
    const chain = (await await chainStorageUtils.getData(`active_chain`)) as Chain;
    if (chain) {
      return chain;
    }
    const chains = await chainStorageUtils.getChains();
    return chains[0];
  }
}

export const settingStoreUtils = {
  getWindowId: () => localForgeUtils.getData(settingStore, `window_id`),
  setWindowId: (id: number) => localForgeUtils.setData(settingStore, `window_id`, id, false),

  setDebug: (debug: boolean) => {
    localStorage.setItem('debug', String(debug));
  },
  getDebug: () => localStorage.getItem('debug') === 'true',
  getColor: () => localForgeUtils.getData(settingStore, `account-color`),
  setColor: (color: string) => localForgeUtils.setData(settingStore, `account-color`, color, false),
};

export interface SubscribeContractItem {
  accountId?: string;
  contractName: string;
  contractIcon?: string;
  contractType: ContractType;
  FTPerName?: string;
  FTMinPrecision?: number;
  balance?: string;
}

export interface ContractTxItem {
  txId: string;
  method: string;
  timestamp?: number;
  from?: string;
  to?: string;
  amount?: string;
  height?: number;
  params?: string;
  operator?: string;
  contractType: ContractType;
}

export interface NFTMetaData {
  /**
   * @description 作者名称
   */
  auther?: string;
  /**
   * @description 发行机构
   */
  orgName?: string;
  /**
   * @description 发行品类型， 支持图片，视频，音频
   * @default 'image'
   */
  seriesType?: 'image' | 'video' | 'audio';
  /**
   * @description 发行品名称
   */
  name?: string;
  /**
   * @description 发行品描述
   */
  description?: string;
  /**
   * @description 发行品地址
   */
  image: string;
  /**
   * @description 发行品hash
   */
  seriesHash?: string;
}

export interface ContractNFTItem extends NFTMetaData {
  tokenId: string;
}

// 删除用户、删除链，添加订阅数据时发现账户不存在， 则取消当前账号下的订阅
// 取消订阅时 keys
const txMaxLength = 1000;
export const contractStoreUtils = {
  async getSubscribe(chainId?: string, contractName?: string) {
    const subscribeList: SubscribeContractItem[] = (await contractStore.getItem(`${chainId}_subscribe_list`)) || [];
    if (contractName) {
      return subscribeList.filter((ele) => {
        if (ele.contractName === contractName) {
          // 类型转换，兼容历史类型 FT,NFT
          if (CONTRACT_TYPE_REPLACE_MAP[ele.contractType]) {
            // eslint-disable-next-line no-param-reassign
            ele.contractType = CONTRACT_TYPE_REPLACE_MAP[ele.contractType];
          }
          return true;
        }
        return false;
      });
    }

    return subscribeList;
  },
  async setSubscribe(chainId: string, contractInfo: SubscribeContractItem) {
    // {contractName,contractType,FTPerName,FTMinPrecision}
    const subscribeList = await this.getSubscribe(chainId);
    let repeat = '';
    subscribeList.forEach((ele) => {
      if (ele.contractName === contractInfo.contractName) {
        console.warn('合约名称重复: contractName', contractInfo.contractName);
        repeat = contractInfo.contractName;
      }
    });
    if (repeat) {
      throw new Error(`合约名称重复,contractName:${repeat}`);
    }
    subscribeList.push(contractInfo);

    return contractStore.setItem(`${chainId}_subscribe_list`, subscribeList);
  },
  async abortSubscribe(chainId: string, contractName?: string) {
    // 取消合约订阅
    if (contractName) {
      const subscribeList = await this.getSubscribe(chainId);
      let has;
      let index;
      subscribeList.forEach((ele, i) => {
        if (ele.contractName === contractName) {
          has = true;
          index = i;
        }
      });
      if (!has) {
        return console.warn('合不存在: contractName', contractName);
      }
      subscribeList.splice(index, 1);
      // 清除链合约数据
      const keys = await contractStore.keys();
      for (const key of keys) {
        if (new RegExp(`^${chainId}_${contractName}_`).test(key)) {
          await contractStore.removeItem(key);
        }
      }
      // 更新订阅列表
      return contractStore.setItem(`${chainId}_subscribe_list`, subscribeList);
    }
    // 清除账号，订阅数据， 所有合约数据
    const keys = await contractStore.keys();
    for (const key of keys) {
      if (new RegExp(`^${chainId}_`).test(key)) {
        await contractStore.removeItem(key);
      }
    }
  },
  // 账户
  async getContractTxs<T extends ContractTxItem>({
    chainId,
    contractName,
    accountId,
    pageSize = 10,
    pageIndex,
  }: {
    chainId: string;
    contractName: string;
    accountId?: string;
    pageSize?: number;
    pageIndex?: number;
  }) {
    let result: T[] = (await contractStore.getItem(`${chainId}_${contractName}_txs`)) || [];
    if (accountId) {
      result = result.filter(
        ({ operator, from, to }) => operator === accountId || from === accountId || to === accountId,
      );
      // 类型转换，兼容历史类型 FT,NFT
      result.forEach((ele) => {
        if (CONTRACT_TYPE_REPLACE_MAP[ele.contractType]) {
          // eslint-disable-next-line no-param-reassign
          ele.contractType = CONTRACT_TYPE_REPLACE_MAP[ele.contractType];
        }
      });
    }
    if (pageIndex === undefined) {
      return result;
    }
    return result.splice(pageSize * pageIndex, pageSize);
  },
  async addContractTxs({
    chainId,
    contractName,
    txList,
  }: {
    chainId: string;
    contractName: string;
    txList: ContractTxItem[];
  }) {
    // 判断是否已经订阅
    const subscribeList = await this.getSubscribe(chainId, contractName);
    if (!subscribeList.length) {
      console.debug(`addContractTx 合约${contractName} 未订阅`);
      return;
    }
    const txSourceList: ContractTxItem[] = await this.getContractTxs({ contractName, chainId });
    txList.forEach((ele) => {
      const repeat = txSourceList.some((item) => item.txId === ele.txId);
      if (!repeat) {
        txSourceList.unshift(ele);
        if (txSourceList.length > txMaxLength) txSourceList.pop();
      }
    });
    return contractStore.setItem(`${chainId}_${contractName}_txs`, txSourceList);
  },

  async getContractBalance({ accountId, chainId, contractName }) {
    const results: string = (await contractStore.getItem(`${chainId}_${contractName}_${accountId}_balance`)) || '';
    return results;
  },
  async setContractBalance({ accountId, chainId, contractName, balance }) {
    const results: ContractNFTItem[] = await contractStore.setItem(
      `${chainId}_${contractName}_${accountId}_balance`,
      balance,
    );
    return results;
  },
  async getContractNFT({ accountId, chainId, contractName }) {
    const results: ContractNFTItem[] =
      (await contractStore.getItem(`${chainId}_${contractName}_${accountId}_nft`)) || [];
    return results;
  },
  async setContractNFT({
    accountId,
    chainId,
    contractName,
    NFTList,
  }: {
    chainId: string;
    accountId: string;
    contractName: string;
    NFTList: ContractNFTItem[];
  }) {
    return await contractStore.setItem(`${chainId}_${contractName}_${accountId}_nft`, NFTList);
  },
};
